"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.withSimpleCachingAsync = void 0;
const type_fns_1 = require("type-fns");
const getCacheFromCacheOption_1 = require("../options/getCacheFromCacheOption");
const defaults_1 = require("../serde/defaults");
/**
 * a wrapper which adds asynchronous caching to asynchronous logic
 *
 * note
 * - utilizes an additional in-memory synchronous cache under the hood to prevent duplicate requests (otherwise, while async cache is resolving, a duplicate parallel request may have be made)
 */
const withSimpleCachingAsync = (logic, { cache: cacheOption, serialize: { key: serializeKey = defaults_1.defaultKeySerializationMethod, // default serialize key to JSON.stringify
value: serializeValue = defaults_1.defaultValueSerializationMethod, // default serialize value to noOp
 } = {}, deserialize: { value: deserializeValue = defaults_1.noOp } = {}, secondsUntilExpiration, }) => {
    return ((...args) => __awaiter(void 0, void 0, void 0, function* () {
        // define key based on args the function was invoked with
        const key = serializeKey({ forInput: args });
        // define cache based on options
        const cache = (0, getCacheFromCacheOption_1.getCacheFromCacheOption)({ forInput: args, cacheOption });
        // see if its already cached
        const cachedValue = yield cache.get(key);
        if ((0, type_fns_1.isNotUndefined)(cachedValue))
            return deserializeValue(cachedValue); // if already cached, return it immediately
        // if its not, grab the output from the logic
        const output = yield logic(...args);
        // set the output to the cache
        const serializedOutput = serializeValue(output);
        yield cache.set(key, serializedOutput, { secondsUntilExpiration });
        // and now re-get from the cache, to ensure that output on first response === output on second response
        const cachedValueNow = yield cache.get(key);
        if ((0, type_fns_1.isNotUndefined)(cachedValueNow))
            return deserializeValue(cachedValueNow);
        // otherwise, somehow, get-after-set returned undefined. warn about this and return output
        // eslint-disable-next-line no-console
        console.warn(
        // warn about this because it should never occur
        'withSimpleCachingAsync encountered a situation which should not occur: cache.get returned undefined immediately after having been set. returning the output directly to prevent irrecoverable failure.', { key });
        return output;
    }));
};
exports.withSimpleCachingAsync = withSimpleCachingAsync;
//# sourceMappingURL=withSimpleCachingAsync.js.map